import os
import numpy as np

import bpy
from bpy.types import Panel
import bpy.utils.previews


from .debug import log, debug_mode
from .mechanist import HVEMechanist
from . import ops

icons = {}


def register_icons():
    pc = bpy.utils.previews.new()
    d = os.path.join(os.path.dirname(__file__), 'icons', )
    pc.load('CLOUD_FILL', os.path.join(d, 'hve_fill.png'), 'IMAGE', )
    pc.load('CLOUD_OUTLINE', os.path.join(d, 'hve_outline.png'), 'IMAGE', )
    icons['main'] = pc


def unregister_icons():
    for pc in icons.values():
        bpy.utils.previews.remove(pc)
    icons.clear()


def preferences():
    a = os.path.split(os.path.split(os.path.realpath(__file__))[0])[1]
    p = bpy.context.preferences.addons[a].preferences
    return p


def update_panel_bl_category(self, context, ):
    main_panels = (HVE_PT_mechanist, )
    sub_panels = (                  
                  HVE_PT_mechanist_export,
                  HVE_PT_mechanist_setup,
                  )
    try:
        for p in main_panels:
            bpy.utils.unregister_class(p)
        for sp in sub_panels:
            bpy.utils.unregister_class(sp)
        prefs = preferences()
        c = prefs.category_custom
        n = ''
        if(c):
            n = prefs.category_custom_name
        else:
            v = prefs.category
            ei = prefs.bl_rna.properties['category'].enum_items
            for e in ei:
                if(e.identifier == v):
                    n = e.name
        if(n == ''):
            raise Exception('Name is empty string')
        for p in main_panels:
            p.bl_category = n
            bpy.utils.register_class(p)
        for sp in sub_panels:
            bpy.utils.register_class(sp)
    except Exception as e:
        log('HVE setting tab name failed ({})'.format(str(e)))


class HVE_PT_mechanist_base(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Human Vehicle Environment"
    bl_label = "HVEMechanist Base"
    # bl_options = {'DEFAULT_CLOSED'}
    
    def prop_name(self, cls, prop, colon=False, ):
        for p in cls.bl_rna.properties:
            if(p.identifier == prop):
                if(colon):
                    return "{}:".format(p.name)
                return p.name
        return ''
    
    def third_label_two_thirds_prop(self, cls, prop, uil, ):
        f = 0.33
        r = uil.row()
        s = r.split(factor=f)
        s.label(text=self.prop_name(cls, prop, True, ))
        s = s.split(factor=1.0)
        r = s.row()
        r.prop(cls, prop, text='', )
    
    def table_one_third_label_two_thirds_value(self, uil, label, value, colon=True, icon=None, ):
        f = 0.33
        r = uil.row()
        s = r.split(factor=f)
        if(colon):
            s.label(text='{}:'.format(label))
        else:
            s.label(text='{}'.format(label))
        s = s.split(factor=1.0)
        r = s.row()
        if(icon is not None):
            r.label(text='{}'.format(value), icon=icon, )
        else:
            r.label(text='{}'.format(value))    
      
    @classmethod
    def poll(cls, context):
        # o = context.active_object
        # if(o is None):
        #     return False
        # return True
        
        o = context.active_object
        if(o is None):
            return False
        hve = o.human_vehicle_environment
        return True
    def draw(self, context):
        o = context.active_object
        if(o is None):
            self.layout.label(text='Select an object..', icon='ERROR', )
            return
        
        hve = o.human_vehicle_environment
        l = self.layout
        c = l.column()
        c.separator()
        
class HVE_PT_mechanist(HVE_PT_mechanist_base):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Human Vehicle Environment"
    bl_label = "Human Vehicle Environment"
    # bl_parent_id = "HVE_PT_panel"
    # bl_options = {'DEFAULT_CLOSED'}
    
    @classmethod
    def poll(cls, context):
        # o = context.object
        # if(o is None):
        #     return False
        # # if(o):
        # #     hve = o.human_vehicle_environment
        # #     if(hve.edit_is_edit_mesh):
        # #         return False
        # #     if(hve.edit_initialized):
        # #         return False
        return True
    
    def draw_header(self, context):
        # hve = context.object.human_vehicle_environment
        l = self.layout
        # l.label(text='', icon='EXPERIMENTAL', )
        c = icons['main']['CLOUD_OUTLINE']
        o = context.object
        if(o is not None):
            c = icons['main']['CLOUD_FILL']
        l.label(text='', icon_value=c.icon_id, )
    
       

class HVE_PT_mechanist_export(HVE_PT_mechanist_base):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Human Vehicle Environment"
    bl_label = "Export"
    bl_parent_id = "HVE_PT_mechanist"
    # bl_options = {'DEFAULT_CLOSED'}
    
    @classmethod
    def poll(cls, context):
        # o = context.object
        # if(o is None):
        #     return False
        # # if(o):
        # #     hve = o.human_vehicle_environment
        # #     if(hve.edit_is_edit_mesh):
        # #         return False
        # #     if(hve.edit_initialized):
        # #         return False
        return True
    
    def draw(self, context):
        o = context.active_object
        if(o is None):
            self.layout.label(text='Select an object..', icon='ERROR', )
            return
        
        hve = o.human_vehicle_environment
        l = self.layout
        c = l.column()
        r = l.row()
        
        r.label(text="HVE Exporter")
        l.operator("export_scene.iv", text="Export to HVE", icon='EXPORT')

class HVE_PT_mechanist_setup(HVE_PT_mechanist_base):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Human Vehicle Environment"
    bl_label = "Vehicle Setup"
    bl_parent_id = "HVE_PT_mechanist"
    #bl_options = {'DEFAULT_CLOSED'}
    
    def draw(self, context):
        o = context.object
        hve = o.human_vehicle_environment
     
        l = self.layout
        c = l.column()
        
        c.separator()
        c.operator("hve_material.add_hve_material")

        c.separator()
        c.operator("hve_material.add_standard_materials")
                
        c.separator()
        c.operator("hve_material.add_hve_light_materials")           
  
        
        c.separator()       
        
        c.label(text="HVE Light")
        c.prop(hve.make_light, 'type')
        
        


classes = (
    HVE_PT_mechanist,
    HVE_PT_mechanist_export,
    HVE_PT_mechanist_setup,
)
